@@ -7,8 +7,6 @@
 #ifndef FLEXBAR_MULTIPLEXEDREAD_H_
 #define FLEXBAR_MULTIPLEXEDREAD_H_
 
-/* Class represents either a single read or a paired read.
-   In both cases a barcode-read can be also present. */
 
 template <typename TString, typename TIDString>
 class MultiplexedRead {
@@ -22,13 +20,14 @@ public:
 	TSequencingRead *m_b;
 	
 	TString m_randTag;
-	int m_barcode_id;
+	int m_barcode_id, m_barcode_id2;
 	
 	MultiplexedRead(TSequencingRead *r1, TSequencingRead *r2, TSequencingRead *b) :
 		m_r1(r1),
 		m_r2(r2),
 		m_b(b),
 		m_barcode_id(0),
+		m_barcode_id2(0),
 		m_randTag(""){
 	};
 	
