@@ -1,8 +1,7 @@
 /*
  *   Options.h
  *
- *   Created on: Jul 31, 2012
- *       Author: jtr
+ *   Author: jtr
  */
 
 #ifndef FLEXBAR_OPTIONS_H_
@@ -25,11 +24,13 @@ typedef std::pair< SequencingRead<seqan::CharString, seqan::CharString>*,
 
 struct Options{
 	
-	std::string readsFile, readsFile2, barReadsFile, barcodeFile, adapterFile;
+	std::string readsFile, readsFile2, barReadsFile;
+	std::string barcodeFile, adapterFile, barcode2File, adapter2File;
 	std::string adapterSeq, targetName, logLevelStr, outCompression;
 	
-	bool isColorSpace, isPaired, useAdapterFile, useNumberTag, useRemovalTag, useStdin, useStdout;
-	bool switch2Fasta, writeUnassigned, writeSingleReads, writeLengthDist, randTag;
+	bool isColorSpace, isPaired, useAdapterFile, useNumberTag, useRemovalTag, randTag;
+	bool switch2Fasta, writeUnassigned, writeSingleReads, writeLengthDist;
+	bool useStdin, useStdout, relaxRegion, revCompAdapter;
 	
 	int cutLen_begin, cutLen_end, phred_preQual, cutLen_read, a_tail_len, b_tail_len;
 	int maxUncalled, min_readLen, a_min_overlap, b_min_overlap, nThreads;
@@ -46,7 +47,7 @@ struct Options{
 	flexbar::BarcodeDetect   barDetect;
 	flexbar::AdapterRemoval  adapRm;
 	
-	tbb::concurrent_vector<TAdapter> adapters, barcodes;
+	tbb::concurrent_vector<TAdapter> barcodes, adapters, barcodes2, adapters2;
 	
 	std::ostream *out;
 	std::fstream fstrmOut;
@@ -58,6 +59,8 @@ struct Options{
 		barReadsFile   = "";
 		barcodeFile    = "";
 		adapterFile    = "";
+		barcode2File   = "";
+		adapter2File   = "";
 		outCompression = "";
 		
 		isColorSpace     = false;
@@ -72,6 +75,8 @@ struct Options{
 		randTag          = false;
 		useStdin         = false;
 		useStdout        = false;
+		relaxRegion      = false;
+		revCompAdapter   = false;
 		
 		cutLen_begin  = 0;
 		cutLen_end    = 0;
@@ -91,7 +96,7 @@ struct Options{
 };
 
 
-const std::string getFlexbarBanner(const std::string version){
+const std::string getFlexbarBanner(const seqan::CharString version){
 	
 	std::string banner = "";
 	
@@ -102,8 +107,10 @@ const std::string getFlexbarBanner(const std::string version){
 	banner += "           /_/   /_/\\___/_/|_/_.___/\\__,_/_/     \n\n";
 	
 	banner += "Flexbar - flexible barcode and adapter removal, version ";
-	banner += version;
-	banner += "\nBioinformatics in Quantitative Biology at BIMSB, GPLv3\n";
+	
+	append(banner, version);
+	
+	banner += "\n";
 	
 	return banner;
 }
@@ -128,9 +135,11 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	addUsageLine(parser, "\\fB-r\\fP reads [\\fB-t\\fP target] [\\fB-b\\fP barcodes] [\\fB-a\\fP adapters] [options]");
 	
 	// addOption(parser, ArgParseOption("v", "version", "Display program version."));
-	addOption(parser, ArgParseOption("w", "advanced", "Print advanced options help screen."));
+	addOption(parser, ArgParseOption("H", "advanced", "Print help with advanced options."));
+	addOption(parser, ArgParseOption("M", "man", "Print advanced options as man document."));
 	addOption(parser, ArgParseOption("i", "cite", "Show program reference for citation."));
 	
+	// OUTPUTPREFIX
 	addSection(parser, "Basic options");
 	addOption(parser, ArgParseOption("n", "threads", "Number of threads to employ.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("t", "target", "Prefix for output file names or paths.", ARG::STRING));
@@ -140,12 +149,13 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	addOption(parser, ArgParseOption("c", "color-space", "Input in color-space format csfasta or csfastq in sanger scaling."));
 	
 	addSection(parser, "Barcode detection");
-	addOption(parser, ArgParseOption("b",  "barcodes", "Fasta file with barcodes for demultiplexing that may contain N.", ARG::INPUTFILE));
-	addOption(parser, ArgParseOption("br", "barcode-reads", "Fasta/q file composed of separate barcode reads for detection.", ARG::INPUTFILE));
+	addOption(parser, ArgParseOption("b",  "barcodes", "Fasta file with barcodes for demultiplexing, may contain N.", ARG::INPUTFILE));
+	addOption(parser, ArgParseOption("b2", "barcodes2", "Additional barcodes file for second read set in paired mode.", ARG::INPUTFILE));
+	addOption(parser, ArgParseOption("br", "barcode-reads", "Fasta/q file containing separate barcode reads for detection.", ARG::INPUTFILE));
 	addOption(parser, ArgParseOption("be", "barcode-trim-end", "Type of detection, see section trim-end modes.", ARG::STRING));
 	addOption(parser, ArgParseOption("bn", "barcode-tail-length", "Region size in tail trim-end modes. Default: barcode length.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("bo", "barcode-min-overlap", "Minimum overlap of barcode and read. Default: barcode length.", ARG::INTEGER));
-	addOption(parser, ArgParseOption("bt", "barcode-threshold", "Allowed mismatches and gaps per 10 bases overlap.", ARG::DOUBLE));
+	addOption(parser, ArgParseOption("bt", "barcode-threshold", "Allowed mismatches and gaps per overlap of 10.", ARG::DOUBLE));
 	addOption(parser, ArgParseOption("bk", "barcode-keep", "Keep barcodes within reads instead of removal."));
 	addOption(parser, ArgParseOption("bu", "barcode-unassigned", "Include unassigned reads in output generation."));
 	addOption(parser, ArgParseOption("bm", "barcode-match", "Alignment match score.", ARG::INTEGER));
@@ -154,25 +164,19 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	
 	addSection(parser, "Adapter removal");
 	addOption(parser, ArgParseOption("a",  "adapters", "Fasta file with adapters for removal that may contain N.", ARG::INPUTFILE));
+	addOption(parser, ArgParseOption("a2", "adapters2", "File with extra adapters for second read set in paired mode.", ARG::INPUTFILE));
 	addOption(parser, ArgParseOption("as", "adapter-seq", "Single adapter sequence as alternative to adapters option.", ARG::STRING));
+	addOption(parser, ArgParseOption("ar", "adapter-read-set", "Consider only single read set for adapters.", ARG::STRING));
+	addOption(parser, ArgParseOption("ac", "adapter-revcomp", "Consider also reverse complement of each adapter in search."));
 	addOption(parser, ArgParseOption("ae", "adapter-trim-end", "Type of removal, see section trim-end modes.", ARG::STRING));
 	addOption(parser, ArgParseOption("an", "adapter-tail-length", "Region size for tail trim-end modes. Default: adapter length.", ARG::INTEGER));
-	addOption(parser, ArgParseOption("ad", "adapter-read-set", "Restriction to single read set in paired mode.", ARG::STRING));
-	addOption(parser, ArgParseOption("ao", "adapter-min-overlap", "Minimum overlap of adapter and read sequence.", ARG::INTEGER));
-	addOption(parser, ArgParseOption("at", "adapter-threshold", "Allowed mismatches and gaps per 10 bases overlap.", ARG::DOUBLE));
+	addOption(parser, ArgParseOption("ad", "adapter-relaxed", "Skip restriction to pass read ends in right and left modes."));
+	addOption(parser, ArgParseOption("ao", "adapter-min-overlap", "Minimum overlap of adapter and read for removal.", ARG::INTEGER));
+	addOption(parser, ArgParseOption("at", "adapter-threshold", "Allowed mismatches and gaps per overlap of 10.", ARG::DOUBLE));
 	addOption(parser, ArgParseOption("am", "adapter-match", "Alignment match score.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("ai", "adapter-mismatch", "Alignment mismatch score.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("ag", "adapter-gap", "Alignment gap score.", ARG::INTEGER));
 	
-	// addSection(parser, "Joining paired reads");
-	// addOption(parser, ArgParseOption("j",  "join", "Fasta file with adapters for removal that may contain N.", ARG::STRING));
-	// addOption(parser, ArgParseOption("jo", "join-min-overlap", "Minimum overlap of adapter and read sequence.", ARG::INTEGER));
-	// addOption(parser, ArgParseOption("jn", "join-max-overlap", "Region size for tail trim-end modes. Default: adapter length.", ARG::INTEGER));
-	// addOption(parser, ArgParseOption("jt", "join-threshold", "Allowed mismatches and gaps per 10 bases overlap.", ARG::DOUBLE));
-	// addOption(parser, ArgParseOption("jm", "join-match", "Alignment match score.", ARG::INTEGER));
-	// addOption(parser, ArgParseOption("ji", "join-mismatch", "Alignment mismatch score.", ARG::INTEGER));
-	// addOption(parser, ArgParseOption("jg", "join-gap", "Alignment gap score.", ARG::INTEGER));
-	
 	addSection(parser, "Filtering and trimming");
 	addOption(parser, ArgParseOption("u", "max-uncalled", "Allowed uncalled bases (N or .) for each read.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("x", "pre-trim-left", "Trim given number of bases on 5' read end before detection.", ARG::INTEGER));
@@ -182,17 +186,17 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	addOption(parser, ArgParseOption("m", "min-read-length", "Minimum read length to remain after removal.", ARG::INTEGER));
 	
 	addSection(parser, "Output selection");
-	addOption(parser, ArgParseOption("o", "fasta-output", "Prefer non-quality formats fasta and csfasta for output."));
-	addOption(parser, ArgParseOption("z", "zip-output", "Enable direct compression of output files.", ARG::STRING));
-	addOption(parser, ArgParseOption("1", "stdout-reads", "Write reads to stdout instead of files if not being paired."));
+	addOption(parser, ArgParseOption("o", "fasta-output", "Prefer non-quality format fasta or csfasta for output."));
+	addOption(parser, ArgParseOption("z", "zip-output", "Direct compression of output files.", ARG::STRING));
+	addOption(parser, ArgParseOption("1", "stdout-reads", "Write reads to stdout, tagged and interleaved if needed."));
 	addOption(parser, ArgParseOption("j", "length-dist", "Generate length distribution for read output files."));
 	addOption(parser, ArgParseOption("s", "single-reads", "Write single paired reads for too short counterparts."));
 	
 	addSection(parser, "Logging and tagging");
-	addOption(parser, ArgParseOption("l", "log-level", "Print valid optimal read alignment.", ARG::STRING));
+	addOption(parser, ArgParseOption("l", "log-level", "Print chosen read alignments.", ARG::STRING));
 	addOption(parser, ArgParseOption("g", "removal-tags", "Tag reads that are subject to adapter or barcode removal."));
 	addOption(parser, ArgParseOption("e", "number-tags", "Replace read tags by ascending number to save space."));
-	addOption(parser, ArgParseOption("d", "random-tags", "Random read tags at barcode or adapter positions with N."));
+	addOption(parser, ArgParseOption("d", "random-tags", "Capture read sequence at barcode or adapter N positions."));
 	
 	addSection(parser, "Trim-end modes");
 	addText(parser._toolDoc, "ANY: longer side of read remains after removal of overlap", false);
@@ -202,27 +206,50 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	addText(parser._toolDoc, "RIGHT_TAIL: use only last n bases, see tail-length options", false);
 	
 	
+	hideOption(parser, "barcodes2");
 	hideOption(parser, "barcode-tail-length");
 	hideOption(parser, "barcode-keep");
 	hideOption(parser, "barcode-match");
 	hideOption(parser, "barcode-mismatch");
 	hideOption(parser, "barcode-gap");
 	
+	hideOption(parser, "adapters2");
+	hideOption(parser, "adapter-revcomp");
 	hideOption(parser, "adapter-tail-length");
+	hideOption(parser, "adapter-relaxed");
 	hideOption(parser, "adapter-read-set");
 	hideOption(parser, "adapter-match");
 	hideOption(parser, "adapter-mismatch");
 	hideOption(parser, "adapter-gap");
 	
+	hideOption(parser, "man");
 	hideOption(parser, "version");
 	hideOption(parser, "stdout-reads");
 	hideOption(parser, "length-dist");
 	hideOption(parser, "number-tags");
 	hideOption(parser, "random-tags");
 	
+	
+	// setCategory(parser, "Trimming");
 	// setRequired(parser, "reads");
 	// setMinValue(parser, "threads", "1");
 	// setValidValues(parser, "format", "sanger solexa i1.3 i1.5 i1.8");
+	
+	// setValidValues(parser, "target", "fasta fa fastq fq");
+	// setValidValues(parser, "reads", "fasta fa fastq fq");
+	// setValidValues(parser, "reads2", "fasta fa fastq fq");
+	// setValidValues(parser, "barcode-reads", "fasta fa fastq fq");
+	
+	// setValidValues(parser, "barcodes", "fasta fa");
+	// setValidValues(parser, "barcodes2", "fasta fa");
+	// setValidValues(parser, "adapters", "fasta fa");
+	// setValidValues(parser, "adapters2", "fasta fa");
+	
+	// setValidValues(parser, "adapter-trim-end", "ANY LEFT RIGHT LEFT_TAIL RIGHT_TAIL");
+	// setMinValue(parser, "adapter-tail-length", "1");
+	// setMinValue(parser, "adapter-min-overlap", "1");
+	// setMinValue(parser, "adapter-threshold",   "0");
+	// setMaxValue(parser, "adapter-threshold",   "10");
 	// 
 	// setValidValues(parser, "barcode-trim-end", "ANY LEFT RIGHT LEFT_TAIL RIGHT_TAIL");
 	// setMinValue(parser, "barcode-tail-length", "1");
@@ -230,12 +257,6 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	// setMinValue(parser, "barcode-threshold",   "0");
 	// setMaxValue(parser, "barcode-threshold",   "10");
 	// 
-	// setValidValues(parser, "adapter-trim-end", "ANY LEFT RIGHT LEFT_TAIL RIGHT_TAIL");
-	// setMinValue(parser, "adapter-tail-length", "1");
-	// setMinValue(parser, "adapter-min-overlap", "1");
-	// setMinValue(parser, "adapter-threshold",   "0");
-	// setMaxValue(parser, "adapter-threshold",   "10");
-	// 
 	// setMinValue(parser, "max-uncalled",     "0");
 	// setMinValue(parser, "pre-trim-left",    "1");
 	// setMinValue(parser, "pre-trim-right",   "1");
@@ -243,13 +264,14 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	// setMinValue(parser, "post-trim-length", "1");
 	// setMinValue(parser, "min-read-length",   "1");
 	
+	
 	setValidValues(parser, "log-level", "ALL MOD TAB");
 	setValidValues(parser, "zip-output", "GZ BZ2");
 	setValidValues(parser, "adapter-read-set", "1 2");
 	
-	setDefaultValue(parser, "target",         "flexbar");
-	setDefaultValue(parser, "threads",        "1");
-	setDefaultValue(parser, "max-uncalled",   "0");
+	setDefaultValue(parser, "target",          "flexbar");
+	setDefaultValue(parser, "threads",         "1");
+	setDefaultValue(parser, "max-uncalled",    "0");
 	setDefaultValue(parser, "min-read-length", "18");
 	
 	setDefaultValue(parser, "barcode-trim-end",  "ANY");
@@ -259,11 +281,11 @@ void defineOptionsAndHelp(seqan::ArgumentParser &parser, const std::string versi
 	setDefaultValue(parser, "barcode-gap",      "-9");
 	
 	setDefaultValue(parser, "adapter-trim-end",    "RIGHT");
-	setDefaultValue(parser, "adapter-min-overlap", "1");
+	setDefaultValue(parser, "adapter-min-overlap", "3");
 	setDefaultValue(parser, "adapter-threshold",   "3.0");
 	setDefaultValue(parser, "adapter-match",       "1");
 	setDefaultValue(parser, "adapter-mismatch",   "-1");
-	setDefaultValue(parser, "adapter-gap",        "-7");
+	setDefaultValue(parser, "adapter-gap",        "-6");
 	
 	
 	addTextSection(parser, "EXAMPLES");
@@ -300,9 +322,9 @@ void parseCommandLine(seqan::ArgumentParser &parser, std::string version, int ar
 	if(res != ArgumentParser::PARSE_OK){
 		
 		if(isSet(parser, "help")){
-			cout << "\nAdvanced options: flexbar -w\n";
+			cerr << "\nAdvanced options: flexbar -H\n";
 		}
-		cout << endl;
+		cerr << endl;
 		
 		exit(res == ArgumentParser::PARSE_ERROR);
 	}
@@ -312,53 +334,61 @@ void parseCommandLine(seqan::ArgumentParser &parser, std::string version, int ar
 		exit(0);
 	}
 	
-	if(isSet(parser, "advanced")){
+	if(isSet(parser, "advanced") || isSet(parser, "man")){
+		
+		hideOption(parser, "barcodes2",           false);
 		hideOption(parser, "barcode-tail-length", false);
 		hideOption(parser, "barcode-keep",        false);
 		hideOption(parser, "barcode-match",       false);
 		hideOption(parser, "barcode-mismatch",    false);
 		hideOption(parser, "barcode-gap",         false);
 		
+		hideOption(parser, "adapter-revcomp",     false);
 		hideOption(parser, "adapter-tail-length", false);
+		hideOption(parser, "adapter-relaxed",     false);
 		hideOption(parser, "adapter-read-set",    false);
 		hideOption(parser, "adapter-match",       false);
 		hideOption(parser, "adapter-mismatch",    false);
 		hideOption(parser, "adapter-gap",         false);
 		
+		hideOption(parser, "adapters2",    false);
 		hideOption(parser, "version",      false);
 		hideOption(parser, "stdout-reads", false);
 		hideOption(parser, "length-dist",  false);
 		hideOption(parser, "number-tags",  false);
 		hideOption(parser, "random-tags",  false);
 		
-		printHelp(parser);
-		cout << "\nFurther documentation on flexbar.sourceforge.net\n" << endl;
+		if(isSet(parser, "man")) printHelp(parser, cerr, "man");
+		else{
+			printHelp(parser);
+			cerr << "\nFurther documentation on flexbar.sourceforge.net\n" << endl;
+		}
 		exit(0);
 	}
 	
 	if(! isSet(parser, "reads")){
 		printShortHelp(parser);
-		cout << "\nPlease set required input file.\n" << endl;
+		cerr << "\nPlease set required input file.\n" << endl;
 		exit(1);
 	}
 }
 
 
-void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string version){
+void loadProgramOptions(Options &o, seqan::ArgumentParser &parser){
 	
 	using namespace std;
 	using namespace flexbar;
 	
 	ostream *out = o.out;
 	
-	*out << getFlexbarBanner(version) << "\n" << endl;
+	*out << getFlexbarBanner(getVersion(parser)) << "\n" << endl;
 	printLocalTime(o);
 	
 	
 	// basic options
 	
 	getOptionValue(o.targetName, parser, "target");
-	*out << "Target prefix:         " << o.targetName << endl;
+	*out << "Target name:           " << o.targetName << endl;
 	
 	if(isSet(parser, "color-space")){
 		o.isColorSpace = true;
@@ -427,9 +457,6 @@ void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string v
 	
 	if(isSet(parser, "barcodes")){
 		
-		getOptionValue(o.barcodeFile, parser, "barcodes");
-		*out << "Barcode file:          " << o.barcodeFile << endl;
-		
 		if(isSet(parser, "barcode-reads")){
 			getOptionValue(o.barReadsFile, parser, "barcode-reads");
 			*out << "Barcode reads file:    " << o.barReadsFile << endl;
@@ -438,12 +465,23 @@ void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string v
 		}
 		else o.barDetect = WITHIN_READ_REMOVAL;
 		
-		if(o.runType == SINGLE)      o.runType = SINGLE_BARCODED;
+		getOptionValue(o.barcodeFile, parser, "barcodes");
+		*out << "Barcode file:          " << o.barcodeFile << endl;
+		
+		     if(o.runType == SINGLE) o.runType = SINGLE_BARCODED;
 		else if(o.runType == PAIRED) o.runType = PAIRED_BARCODED;
 		
 		if(o.barDetect == WITHIN_READ_REMOVAL && isSet(parser, "barcode-keep")){
 			o.barDetect = WITHIN_READ;
 		}
+		
+		if(isSet(parser, "barcodes2") && o.barDetect != BARCODE_READ && o.isPaired){
+			getOptionValue(o.barcode2File, parser, "barcodes2");
+			*out << "Barcode file 2:        " << o.barcode2File << endl;
+			
+			if(o.barDetect == WITHIN_READ_REMOVAL) o.barDetect = WITHIN_READ_REMOVAL2;
+			else if(o.barDetect == WITHIN_READ)    o.barDetect = WITHIN_READ2;
+		}
 	}
 	
 	if(isSet(parser, "adapters")){
@@ -456,6 +494,12 @@ void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string v
 		getOptionValue(o.adapterSeq, parser, "adapter-seq");
 		o.adapRm = NORMAL;
 	}
+	
+	if(isSet(parser, "adapters2") && o.adapRm == NORMAL && o.isPaired){
+		getOptionValue(o.adapter2File, parser, "adapters2");
+		*out << "Adapter file 2:        " << o.adapter2File << endl;
+		o.adapRm = NORMAL2;
+	}
 	*out << endl;
 	
 	
@@ -538,11 +582,11 @@ void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string v
 		}
 	}
 	
-	if(isSet(parser, "single-reads")) o.writeSingleReads = true;
-	if(isSet(parser, "length-dist"))  o.writeLengthDist  = true;
-	if(isSet(parser, "number-tags"))  o.useNumberTag     = true;
-	if(isSet(parser, "removal-tags")) o.useRemovalTag    = true;
-	if(isSet(parser, "random-tags"))  o.randTag          = true;
+	if(isSet(parser, "single-reads"))  o.writeSingleReads = true;
+	if(isSet(parser, "length-dist"))   o.writeLengthDist  = true;
+	if(isSet(parser, "number-tags"))   o.useNumberTag     = true;
+	if(isSet(parser, "removal-tags"))  o.useRemovalTag    = true;
+	if(isSet(parser, "random-tags"))   o.randTag          = true;
 	
 	*out << endl;
 	
@@ -623,8 +667,17 @@ void loadProgramOptions(Options &o, seqan::ArgumentParser &parser, std::string v
 			*out << "adapter-tail-length:   " << o.a_tail_len << endl;
 		}
 		
-		if(isSet(parser, "adapter-read-set") && o.isPaired){
-			
+		if(isSet(parser, "adapter-revcomp")){
+			*out << "adapter-revcomp:       yes" << endl;
+			o.revCompAdapter = true;
+		}
+		
+		if(isSet(parser, "adapter-relaxed")){
+			*out << "adapter-relaxed:       yes" << endl;
+			o.relaxRegion = true;
+		}
+		
+		if(isSet(parser, "adapter-read-set") && o.isPaired && o.adapRm != NORMAL2){
 			string a_read_set;
 			getOptionValue(a_read_set, parser, "adapter-read-set");
 			*out << "adapter-read-set:      " << a_read_set << endl;
